/*
_________________________________________________
U S I N G   P O I N T E R S   T O   H E A P   O B J E C T S

Ron Kessler
Created 3/2/2017

_________________________________________________

This app shows a reason to use pointers when we need to store
something in RAM for the lifetime of our app.

When we create local variables they are removed from RAM when the
code block they are created in goes out of scope, right? Well the same is
true of objects. If we instantiate an onject inside a function it is stored on the
stack and will be removed when the funtion end, just like our locals.

So, each C++ app has access to the heap or free store. This is a separate
area of memory that we can use to store things in that must remain available
to our app after a function ends. This is like creating a global variable where we
need it in several places. Items in the heap are removed by the programmer
manually. However, in the lastest versions of Windows, the O/S WILL remove
all the RAM that our app was using in the app domain. So memory leaks are not
nearly as common as they used to be.

In this example if we do not clean up our objects, they can remain in the heap
without any reference. This is called garbage and that is wher the term comes from.


To create an object in the heap we use the 'new' keyword and we create a pointer
to the new object. Then we work with it using our pointer. Never a reference, only
a pointer.

If you forget to delete to delete the pointer, you create a memory leak because your
object cannot be accessed any more. Doing this a lot can make you run out of available
memory.

Notice I forced an exception to show what happens when the delete command can't run
in the try block. This will leave garabge in the heap that you cannot access any more.
One way to handle this is to use the code in the catch block.

*/

#include "stdafx.h"
#include <exception>
#include <iostream>

using namespace std;

int main()
{
	int* p = nullptr;							//create our pointer

	try
		{
		p = { new int{ 1 } };			//create a new int on the heap using a int type pointer
		*p = 2;									//now increment the pointer to 2

		//but lets say an exception occurred here. The delete command won't run and now you have left
		//garabge in memory! You have leaked the memory because you cannot access p after the program crashes.
		throw (exception{ "Big Problem" } );		//simulate an exception
		delete p;
		}
	catch (const exception& e)				//const means this block will not change the exception
		{
			cout << "Exception caught: " << e.what () << "\n\n";			//what is like 'message'

			if (p != nullptr)			//this is what you have to do to delete p if there is an exception
				delete p;				//filling catch blocks like this is discouraged. I will show you a better way soon!
		}
	return 0;
}

